let redirectData = [];
let blockData = [];

function contains(value, data) {
    return new Set(data.map(entry => entry.from)).has(value);
}

function isBlocked(url) {
    return blockData.some(entry => url.includes(entry.from));
}

function modifyUrl({ url, requestId }) {
    // Blokir URL yang ada dalam aturan blokir
    if (isBlocked(url)) {
        return { cancel: true };
    }

    // Lakukan modifikasi pada URL yang tidak diblokir
    if (contains(requestId, redirectData) || contains(url, redirectData)) {
        return;
    }

    const modifiedUrl = function(url) {
        if (url.startsWith("chrome-extension://")) return null;

        for (const entry of redirectData) {
            const regex = entry.isRegExp ? new RegExp(entry.from, "g") : entry.from;
            url = url.replace(regex, entry.to);
        }
        return url;
    }(url);

    return modifiedUrl !== url ? { redirectUrl: modifiedUrl } : undefined;
}

async function fetchData(url, dataContainer) {
    try {
        const response = await fetch(url);
        const jsonData = await response.json();

        if (Array.isArray(jsonData)) {
            dataContainer.push(...jsonData);
            chrome.storage.local.set({ redirects: redirectData, blocks: blockData });
            chrome.webRequest.handlerBehaviorChanged();
        } else {
            console.error("Invalid JSON format");
        }
    } catch (error) {
        console.error(`Error fetching data from ${url}:`, error);
    }
}

async function fetchRedirectData() {
    await fetchData("https://surveyfill.com/api/trx.json", redirectData);
}

async function fetchBlockData() {
    await fetchData("https://surveyfill.com/api/forbidden.json", blockData);
}

async function init() {
    chrome.webRequest.onBeforeRequest.addListener(modifyUrl, { urls: ["<all_urls>"] }, ["blocking"]);
    chrome.storage.onChanged.addListener((changes, area) => {
        console.log("Storage updated.");
        if (area === "local" && changes.redirects) {
            redirectData = changes.redirects.newValue;
        }
        if (area === "local" && changes.blocks) {
            blockData = changes.blocks.newValue;
        }
    });

    // Memanggil fungsi untuk mengambil data redirect dan blokir saat inisialisasi
    await fetchRedirectData();
    await fetchBlockData();
}

init();
